"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AssetsPlugin = require("assets-webpack-plugin");
const chalk_1 = require("chalk");
const CleanWebpackPlugin = require("clean-webpack-plugin");
const fs = require("fs");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const lodash_1 = require("lodash");
const moment = require("moment");
const path = require("path");
const TerserPlugin = require("terser-webpack-plugin");
const webpack = require("webpack");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const get_bundles_info_1 = require("../../components/project_deps/get_bundles_info");
const pkg_1 = require("../../config/pkg");
const const_1 = require("../../const");
const helpers_1 = require("../../helpers");
const index_1 = require("./../../config/analyze/index");
// tslint:disable:variable-name
exports.DllPlugin = (envConfig) => {
    return [
        ...exports.clean(envConfig, [helpers_1.dll_output_path()], true),
        new webpack.DllPlugin({
            context: helpers_1.get_context(),
            name: helpers_1.dll_entry_name(),
            path: helpers_1.dll_manifest_path(),
        }),
    ];
};
exports.DllReferencePlugin = (envConfig, rearguardConfig, exclude_them_self = false) => {
    const bundlesInfo = get_bundles_info_1.get_bundles_info(envConfig, rearguardConfig);
    const plugins = [];
    for (const { has_dll, bundle_entry_name, manifest, load_on_demand } of bundlesInfo) {
        if (!load_on_demand && has_dll) {
            if (path.isAbsolute(manifest) && fs.existsSync(manifest)) {
                plugins.push(new webpack.DllReferencePlugin({
                    context: helpers_1.get_context(),
                    manifest,
                    name: bundle_entry_name.dll,
                }));
            }
            else {
                console.log(chalk_1.default.red(`[ DllReferencePlugin ][ ERROR ][ MANIFEST_NOT_FOUND: ${manifest} ]`));
                process.exit(1);
            }
        }
    }
    // exclude_them_self - выставляется в true в случае когда dll зависит от dll.
    if (!exclude_them_self) {
        if (fs.existsSync(helpers_1.dll_manifest_path())) {
            plugins.push(new webpack.DllReferencePlugin({
                context: helpers_1.get_context(),
                manifest: helpers_1.dll_manifest_path(),
                name: helpers_1.dll_entry_name(),
            }));
        }
    }
    return plugins;
};
class ComputeDataForHWP {
    constructor(envConfig, rearguardConfig) {
        this.rearguardConfig = rearguardConfig;
        this.envConfig = envConfig;
        this.apply = this.apply.bind(this);
    }
    apply(compiler) {
        compiler.hooks.compilation.tap("compute_data_for_html_webpack_plugin", (compilation) => {
            HtmlWebpackPlugin.getHooks(compilation).beforeAssetTagGeneration.tapAsync("compute_data_for_html_webpack_plugin", // <-- Set a meaningful name here for stacktraces
            (compilation_data, callback) => {
                const bundlesInfo = get_bundles_info_1.get_bundles_info(this.envConfig, this.rearguardConfig);
                const data = { js: [], css: [] };
                for (const { assets, bundle_name, has_dll, has_browser_lib, load_on_demand } of bundlesInfo) {
                    if (!load_on_demand) {
                        if (has_dll && fs.existsSync(assets.dll)) {
                            data.js.push(require(assets.dll)[helpers_1.dll_entry_name(bundle_name)].js);
                        }
                        if (has_browser_lib && fs.existsSync(assets.lib)) {
                            data.js.push(require(assets.lib)[helpers_1.lib_entry_name(bundle_name)].js);
                        }
                    }
                }
                if (!this.rearguardConfig.load_on_demand && fs.existsSync(helpers_1.dll_assets_path())) {
                    data.js.push(require(helpers_1.dll_assets_path())[helpers_1.dll_entry_name()].js);
                }
                // Manipulate the content
                compilation_data.assets.js = [...data.js, ...compilation_data.assets.js];
                console.log("");
                console.log(chalk_1.default.bold.green("[ HtmlWebpackPlugin ][ BUILD ][ index.html ]"));
                for (const css of compilation_data.assets.css) {
                    console.log(chalk_1.default.green(`[ CSS ][ ${css} ]`));
                }
                for (const js of compilation_data.assets.js) {
                    console.log(chalk_1.default.green(`[ JS ][ ${js} ]`));
                }
                console.log("");
                // Tell webpack to move on
                callback(null, compilation_data);
            });
        });
    }
}
exports.htmlWebpackPlugin = (envConfig, rearguardConfig) => {
    const { isWDS, isBuild } = envConfig;
    if (isWDS || isBuild) {
        return [
            new HtmlWebpackPlugin({
                filename: "index.html",
                inject: false,
                template: path.resolve(__dirname, "../../../../templates/html-webpack-template.ejs"),
                // tslint:disable-next-line:object-literal-sort-keys
                meta: { viewport: "width=device-width, initial-scale=1, shrink-to-fit=no" },
            }),
            new ComputeDataForHWP(envConfig, rearguardConfig),
        ];
    }
    return [];
};
exports.HMR = (envConfig) => {
    const { isDevelopment, isBuild } = envConfig;
    if (isDevelopment && !isBuild) {
        return [
            // prints more readable module names in the browser console on HMR updates
            new webpack.NamedModulesPlugin(),
            // enable HMR globally
            new webpack.HotModuleReplacementPlugin(),
        ];
    }
    return [];
};
exports.uglify = (envConfig) => {
    const { isDevelopment } = envConfig;
    if (!isDevelopment) {
        return [
            // https://github.com/webpack-contrib/terser-webpack-plugin
            new TerserPlugin({
                cache: true,
                parallel: true,
                terserOptions: {
                    compress: {
                        sequences: true,
                        unused: true,
                    },
                    ecma: 8,
                },
            }),
        ];
    }
    return [];
};
// Webpack Bundle Analyzer
// https://github.com/th0r/webpack-bundle-analyzer
exports.analyze = (envConfig) => {
    const { isDebug } = envConfig;
    const { port } = index_1.analyzeConfig;
    if (isDebug) {
        return [
            new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                analyzerPort: port,
            }),
        ];
    }
    return [];
};
exports.clean = (envConfig, toRemove = [], force = false) => {
    const { isDevelopment, isBuild, isDebug } = envConfig;
    if (!isDevelopment || force || isBuild) {
        return [new CleanWebpackPlugin(toRemove, { root: process.cwd(), verbose: isDebug })];
    }
    return [];
};
// tslint:disable:variable-name
exports.assetsPlugin = (envConfig, bundle_dir) => {
    return [
        new AssetsPlugin({
            path: path.resolve(envConfig.rootDir, bundle_dir, lodash_1.snakeCase(pkg_1.pkgInfo.name), const_1.BUNDLE_SUB_DIR()),
            // tslint:disable-next-line:object-literal-sort-keys
            filename: const_1.ASSETS_NAME,
            processOutput(data) {
                const result = {};
                for (const key in data) {
                    if (key.indexOf(lodash_1.snakeCase(pkg_1.pkgInfo.name)) !== -1) {
                        result[key] = data[key];
                    }
                }
                return JSON.stringify(result, null, 2);
            },
        }),
    ];
};
// tslint:disable-next-line: max-classes-per-file
class HashWebpackPlugin {
    constructor(buildStatusConfig, envConfig) {
        this.buildStatusConfig = buildStatusConfig;
        this.envConfig = envConfig;
        this.apply = this.apply.bind(this);
    }
    apply(compiler) {
        // ! See https://webpack.js.org/api/plugins/compiler/#event-hooks
        compiler.plugin("after-emit", (compilation, callback) => {
            const hash = compilation.hash;
            let hash_key = "hash_dev";
            if (this.envConfig.isDevelopment) {
                hash_key = "hash_dev";
            }
            else {
                hash_key = "hash_prod";
            }
            const cur_hash = this.buildStatusConfig[hash_key];
            if (cur_hash !== hash || this.envConfig.needUpdateBuildTime) {
                this.buildStatusConfig.last_build_time = moment();
            }
            this.buildStatusConfig[hash_key] = hash;
            callback(null);
        });
    }
}
exports.HashWebpackPlugin = HashWebpackPlugin;
// tslint:enable:variable-name
